@@ -8,6 +8,9 @@ class LockingDict():
         self.dict = {}
         self.lock = threading.Lock()
 
+    def get_dict(self):
+        return self.dict
+
     def acquire_lock(self):
         self.lock.acquire()
 
