@@ -231,25 +231,13 @@ class OplogThread(threading.Thread):
             logging.info('Finished rollback')
             return self.get_oplog_cursor(timestamp)
 
-    def get_last_oplog_timestamp(self):
-        """Return the timestamp of the latest entry in the oplog.
-        """
-        curr = self.oplog.find().sort('$natural', pymongo.DESCENDING).limit(1)
-        if curr.count(with_limit_and_skip=True) == 0:
-            return None
-
-        return curr[0]['ts']
-
-    def dump_collection(self, timestamp):
-        """Dumps collection into backend engine.
+    def dump_collection(self):
+        """Dumps collection into the target system.
 
         This method is called when we're initializing the cursor and have no
         configs i.e. when we're starting for the first time.
         """
 
-        if timestamp is None:
-            return None
-
         dump_set = self.namespace_set
 
         #no namespaces specified
@@ -265,16 +253,62 @@ class OplogThread(threading.Thread):
                     namespace = str(db) + "." + str(coll)
                     dump_set.append(namespace)
 
+        long_ts = None
+
         for namespace in dump_set:
             db, coll = namespace.split('.', 1)
             target_coll = self.main_connection[db][coll]
             cursor = util.retry_until_ok(target_coll.find)
-            long_ts = util.bson_ts_to_long(timestamp)
+            cursor = cursor.sort('$natural', pymongo.DESCENDING)
+            oplog_cursor = util.retry_until_ok(self.oplog.find)
+            oplog_cursor = oplog_cursor.sort('$natural', pymongo.DESCENDING)
 
-            for doc in cursor:
-                doc['ns'] = namespace
-                doc['_ts'] = long_ts
-                self.doc_manager.upsert(doc)
+            for entry in oplog_cursor:
+
+                if entry['op'] != 'i':
+                    continue
+                #The 'o' field represents the document
+                search_doc = entry['o']
+                cursor.rewind()
+                for doc in cursor:
+                    if search_doc == doc:
+                        long_ts = util.bson_ts_to_long(entry['ts'])
+                        break
+
+                if long_ts:
+                    break
+
+            cursor.rewind()
+
+            try:
+                for doc in cursor:
+                    doc['ns'] = namespace
+                    doc['_ts'] = long_ts
+                    self.doc_manager.upsert(doc)
+            except (pymongo.errors.AutoReconnect,
+                    pymongo.errors.OperationFailure):
+
+                err_msg = "OplogManager: Failed during dump collection"
+                effect = "cannot recover!"
+                logging.error('%s %s %s' % (err_msg, effect, self.oplog))
+                self.running = False
+                return
+
+        if long_ts:
+            long_ts = util.long_to_bson_ts(long_ts)
+        else:  # Implies that we are just initiating the set
+            long_ts = self.get_last_oplog_timestamp()
+
+        return long_ts
+
+    def get_last_oplog_timestamp(self):
+        """Return the timestamp of the latest entry in the oplog.
+        """
+        curr = self.oplog.find().sort('$natural', pymongo.DESCENDING).limit(1)
+        if curr.count(with_limit_and_skip=True) == 0:
+            return None
+
+        return curr[0]['ts']
 
     def init_cursor(self):
         """Position the cursor appropriately.
@@ -285,10 +319,10 @@ class OplogThread(threading.Thread):
         timestamp = self.read_last_checkpoint()
 
         if timestamp is None:
-            timestamp = util.retry_until_ok(self.get_last_oplog_timestamp)
-            self.dump_collection(timestamp)
-            logging.info('OplogManager: %s Dumped collection into backend'
-                         % self.oplog)
+            timestamp = self.dump_collection()
+            msg = "Dumped collection into target system"
+            logging.info('OplogManager: %s %s'
+                         % (self.oplog, msg))
 
         self.checkpoint = timestamp
         cursor = self.get_oplog_cursor(timestamp)
@@ -301,7 +335,8 @@ class OplogThread(threading.Thread):
         """Store the current checkpoint in the oplog progress dictionary.
         """
         self.oplog_progress.acquire_lock()
-        self.oplog_progress.dict[str(self.oplog)] = self.checkpoint
+        oplog_progress_dict = self.oplog_progress.get_dict()
+        oplog_progress_dict[str(self.oplog)] = self.checkpoint
         self.oplog_progress.release_lock()
 
     def read_last_checkpoint(self):
@@ -310,21 +345,22 @@ class OplogThread(threading.Thread):
         oplog_str = str(self.oplog)
 
         self.oplog_progress.acquire_lock()
-
-        if oplog_str in self.oplog_progress.dict.keys():
+        oplog_dict = self.oplog_progress.get_dict()
+        if oplog_str in oplog_dict.keys():
+            ret_val = oplog_dict[oplog_str]
             self.oplog_progress.release_lock()
-            return self.oplog_progress.dict[oplog_str]
+            return ret_val
         else:
             self.oplog_progress.release_lock()
             return None
 
     def rollback(self):
-        """Rollback backend engine to consistent state.
+        """Rollback target system to consistent state.
 
-        The strategy is to find the latest timestamp in the backend and
-        the largest timestamp in the oplog less than the latest backend
+        The strategy is to find the latest timestamp in the target system and
+        the largest timestamp in the oplog less than the latest target system
         timestamp. This defines the rollback window and we just roll these
-        back until the oplog and backend are in consistent states.
+        back until the oplog and target system are in consistent states.
         """
         self.doc_manager.commit()
         last_inserted_doc = self.doc_manager.get_last_doc()
@@ -332,8 +368,8 @@ class OplogThread(threading.Thread):
         if last_inserted_doc is None:
             return None
 
-        backend_ts = util.long_to_bson_ts(last_inserted_doc['_ts'])
-        last_oplog_entry = self.oplog.find_one({'ts': {'$lte': backend_ts}},
+        target_ts = util.long_to_bson_ts(last_inserted_doc['_ts'])
+        last_oplog_entry = self.oplog.find_one({'ts': {'$lte': target_ts}},
                                                sort=[('$natural',
                                                pymongo.DESCENDING)])
         if last_oplog_entry is None:
@@ -361,7 +397,7 @@ class OplogThread(threading.Thread):
             retry = util.retry_until_ok
             to_update = retry(self.main_connection[db][coll].find,
                               {'_id': {'$in': bson_obj_id_list}})
-            #doc list are docs in backend engine, to_update are docs in mongo
+            #doc list are docs in  target system, to_update are docs in mongo
             doc_hash = {}  # hash by _id
             for doc in doc_list:
                 doc_hash[bson.objectid.ObjectId(doc['_id'])] = doc
